/** 
	A "hello" program in Qt
	that use OpenGL drawing primitives
	File: quad.cpp

	Fraga 19/09/2001
**/
#include <qpainter.h>
#include <ctype.h>
#include "triangle.h"

Triangle::Triangle( QWidget *parent )
		: QGLWidget( parent )
//  : QGLWidget( QGLFormat( QGL::SingleBuffer|QGL::NoDepthBuffer ), parent )
{
	signo = 1;	

	// setMouseTracking( TRUE );

	// setFocusPolicy( QWidget::StrongFocus);
	setFocusPolicy( Qt::StrongFocus);

	timer = new QTimer( this );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	xrot = yrot = 0;
	angle = 0;

	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}


void Triangle::timerDone( )
{
	angle += 10.0*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );
}

void Triangle::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
	xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

void Triangle::keyPressEvent ( QKeyEvent * e )
{
	switch ( e->key() ) {
		case '+':   
			signo = 1;
			break;
		case '-':  
			signo = -1;
			break;
	}
}

//
// This method puts the initial coordinates each time window is resized
//
void Triangle::resizeGL( int l, int a )
{
	float aspect=(float)l/(float)a;

	windowWidth = l;
	windowHeight = a;

	glViewport (0, 0, (GLsizei) l, (GLsizei) a);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	float x = 5.0;
	float y = 5.0;

	if( l <= a )
		glOrtho ( -x, x, -y/aspect, y/aspect, -10.0, 10.0);
	else
		glOrtho ( -x*aspect, x*aspect, -y, y, -10.0, 10.0);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */

	emit changeSize ( );
}

QSizePolicy Triangle::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Triangle::paintGL(void)
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix( );
	glRotatef (yrot, 0.0, 1.0, 0.0);
	glRotatef (xrot, 1.0, 0.0, 0.0);


	glRotatef ( angle , 0.0, 0.0, 1.0);

	// glBegin( GL_LINE_LOOP );
	glTranslatef( 0.0, 1.0, 0.0 );
	glBegin( GL_QUADS );
		glColor3f( 0.0, 0.0, 1.0 );
		glVertex2i(-3, 2 );
		glVertex2i(-3,-4 );
		glVertex2i( 3,-4 );
		glVertex2i( 3, 2 );
	glEnd( );
	glPopMatrix( );

	// swapBuffers( );
}

void Triangle::initializeGL( ) 
{
	glClearColor ( 0.8, 0.8, 0.8, 0.0);	// Background to a grey tone

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

	glEnable (GL_DEPTH_TEST);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */
}


